'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.normalizePath = normalizePath;
exports.makeAbsolute = makeAbsolute;
exports.extract = extract;
exports.extractSync = extractSync;

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _nodeSass = require('node-sass');

var _nodeSass2 = _interopRequireDefault(_nodeSass);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _load = require('./load');

var _process = require('./process');

var _importer = require('./importer');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_bluebird2.default.promisifyAll(_nodeSass2.default);

var NORMALIZED_PATH_SEPARATOR = '/';
var PLATFORM_PATH_SEPARATOR = _path2.default.sep;

/**
 * Normalize path across platforms
 */
function normalizePath(path) {
  return path.split(PLATFORM_PATH_SEPARATOR).join(NORMALIZED_PATH_SEPARATOR);
}

function makeAbsolute(maybeRelativePath) {
  if (_path2.default.posix.isAbsolute(maybeRelativePath)) {
    return maybeRelativePath;
  } else {
    return _path2.default.posix.join(process.cwd(), maybeRelativePath);
  }
}

/**
 * Get rendered stats required for extraction
 */
function getRenderedStats(rendered, compileOptions) {
  return {
    entryFilename: normalizePath(rendered.stats.entry),
    includedFiles: rendered.stats.includedFiles.map(function (f) {
      return normalizePath(makeAbsolute(f));
    }),
    includedPaths: (compileOptions.includePaths || []).map(normalizePath)
  };
}

/**
 * Make the compilation option for the extraction rendering
 * Set the data to be rendered to the injected source
 * Add compilation functions and custom importer for injected sources
 */
function makeExtractionCompileOptions(compileOptions, entryFilename, extractions, importer) {
  var extractionCompileOptions = Object.assign({}, compileOptions);
  var extractionFunctions = {};

  // Copy all extraction function for each file into one object for compilation
  Object.keys(extractions).forEach(function (extractionKey) {
    Object.assign(extractionFunctions, extractions[extractionKey].injectedFunctions);
  });

  extractionCompileOptions.functions = Object.assign(extractionFunctions, compileOptions.functions);
  extractionCompileOptions.data = extractions[entryFilename].injectedData;
  if (!makeExtractionCompileOptions.imported) {
    extractionCompileOptions.importer = importer;
  }

  return extractionCompileOptions;
}

/**
 * Compile extracted variables per file into a complete result object
 */
function compileExtractionResult(extractions) {
  var extractedVariables = { global: {} };

  Object.keys(extractions).map(function (filename) {
    var globalFileVariables = extractions[filename].variables.global;

    Object.keys(globalFileVariables).map(function (variableKey) {
      var variable = extractedVariables.global[variableKey];

      if (!variable) {
        variable = extractedVariables.global[variableKey] = Object.assign({}, globalFileVariables[variableKey].value, {
          sources: [],
          expressions: []
        });
      }

      variable.sources.push(filename);
      variable.expressions.push(globalFileVariables[variableKey].expression);
    });
  });

  return extractedVariables;
}

/**
 * Extract the variables from already rendered sass file(s)
 * Returns the extracted variables
 */
function extract(rendered) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$compileOptions = _ref.compileOptions,
      compileOptions = _ref$compileOptions === undefined ? {} : _ref$compileOptions;

  var _getRenderedStats = getRenderedStats(rendered, compileOptions),
      entryFilename = _getRenderedStats.entryFilename,
      includedFiles = _getRenderedStats.includedFiles,
      includedPaths = _getRenderedStats.includedPaths;

  return (0, _load.loadCompiledFiles)(includedFiles, entryFilename, compileOptions.data).then(function (compiledFiles) {
    var extractions = (0, _process.processFiles)(compiledFiles);
    var importer = (0, _importer.makeImporter)(extractions, includedFiles, includedPaths);
    var extractionCompileOptions = makeExtractionCompileOptions(compileOptions, entryFilename, extractions, importer);

    return _nodeSass2.default.renderAsync(extractionCompileOptions).then(function () {
      return compileExtractionResult(extractions);
    });
  });
}

/**
 * Synchronously extract the variables from already rendered sass file(s)
 * Returns the extracted variables
 */
function extractSync(rendered) {
  var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref2$compileOptions = _ref2.compileOptions,
      compileOptions = _ref2$compileOptions === undefined ? {} : _ref2$compileOptions;

  var _getRenderedStats2 = getRenderedStats(rendered, compileOptions),
      entryFilename = _getRenderedStats2.entryFilename,
      includedFiles = _getRenderedStats2.includedFiles,
      includedPaths = _getRenderedStats2.includedPaths;

  var compiledFiles = (0, _load.loadCompiledFilesSync)(includedFiles, entryFilename, compileOptions.data);
  var extractions = (0, _process.processFiles)(compiledFiles);
  var importer = (0, _importer.makeSyncImporter)(extractions, includedFiles, includedPaths);
  var extractionCompileOptions = makeExtractionCompileOptions(compileOptions, entryFilename, extractions, importer);

  _nodeSass2.default.renderSync(extractionCompileOptions);

  return compileExtractionResult(extractions);
}