'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.makeImporter = makeImporter;
exports.makeSyncImporter = makeSyncImporter;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Search for the imported file in order of included paths
 */
function findImportedPath(url, includedFilesMap, includedPaths) {
  for (var i = 0; i < includedPaths.length; i++) {
    var includedPath = includedPaths[i];

    // Ensure we get absolute included path
    if (!_path2.default.posix.isAbsolute(includedPath)) {
      includedPath = _path2.default.posix.join(process.cwd(), includedPath);
    }

    var candidatePath = _path2.default.posix.join(includedPath, url);

    if (includedFilesMap[candidatePath]) {
      return candidatePath;
    }
  }

  return null;
}

/**
 * Get the absolute file path for a relative @import like './sub/file.scsss'
 * If the @import is made from a raw data section a best guess path is returned
 */
function getImportAbsolutePath(url, prev, includedFilesMap) {
  var includedPaths = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];

  // Ensure that both @import 'file' and @import 'file.scss' is mapped correctly
  var extension = _path2.default.posix.extname(prev);
  if (_path2.default.posix.extname(url) !== extension) {
    url += extension;
  }

  var absolutePath = _path2.default.posix.join(_path2.default.posix.dirname(prev), url);

  if (prev === 'stdin' || !includedFilesMap[absolutePath]) {
    // Find absolute path from included paths as url is not relative to statement origin
    absolutePath = findImportedPath(url, includedFilesMap, includedPaths);

    if (!absolutePath) {
      throw new Error('Can not determine imported file for url \'' + url + '\' imported in ' + prev);
    }
  }

  return absolutePath;
}

/**
 * Get the resulting source and path for a given @import request
 */
function getImportResult(extractions, url, prev, includedFilesMap, includedPaths) {
  var absolutePath = getImportAbsolutePath(url, prev, includedFilesMap, includedPaths);
  var contents = extractions[absolutePath].injectedData;

  return { file: absolutePath, contents: contents };
}

function getIncludedFilesMap(includedFiles) {
  var includedFilesMap = {};
  includedFiles.forEach(function (file) {
    return includedFilesMap[file] = true;
  });
  return includedFilesMap;
}

/**
 * Create an importer that will resolve @import directives with the injected
 * data found in provided extractions object
 */
function makeImporter(extractions, includedFiles, includedPaths) {
  var includedFilesMap = getIncludedFilesMap(includedFiles);

  return function (url, prev, done) {
    try {
      var result = getImportResult(extractions, url, prev, includedFilesMap, includedPaths);
      done(result);
    } catch (err) {
      done(err);
    }
  };
}

/**
 * Create a synchronous importer that will resolve @import directives with the injected
 * data found in provided extractions object
 */
function makeSyncImporter(extractions, includedFiles, includedPaths) {
  var includedFilesMap = getIncludedFilesMap(includedFiles);

  return function (url, prev) {
    try {
      var result = getImportResult(extractions, url, prev, includedFilesMap, includedPaths);
      return result;
    } catch (err) {
      // note: importer must return errors
      return err;
    }
  };
}